USECPM  EQU  0          ;* set to 1 for CP/M, 0 for SOLOS

        ORG  0100H
;******************************************************
;*
;*       <<<  THE GAME OF DEFLECTION  >>>
;*                               BY ANDREW RECUPERO
;*
;* ((( FROM KILOBAUD ISSUE # 14; FEB. 1978 )))
;*
;* ---------------------------------------------------
;* ( EXECUTE PROGRAM AT ADDRESS HEX 0050 )
;* ---------------------------------------------------
;*
;*    TO START:  TYPE IN 0, 1, 2, OR 3 FOR ONE-OF-4 OPTION
;*                 INPUT 0 = 1 TARGET, CLEAR FIELD
;*                   "   1 = 1 TARGET, SHIFT DEFLECTORS
;*                   "   2 = 3 TARGETS, CLEAR FIELD
;*                   "   3 = 3 TARGETS, SHIFT DEFLECTORS
;*
;* THE OBJECT IS TO HIT THE TARGET(S) BY DEFLECTING THE
;*      RUNNER TO THE TARGET.   DEFLECT RUNNER BY
;*      TYPING IN A '/' OR A ' '.   ALL TARGETS MUST BE
;*      HIT WITHIN A FIXED PERIOD OF TIME.
;*
;* ....................................................
;* LINES WITH  A  *@*  SYMBOL IN THE COMMENT FIELD MEAN
;*       THAT THE ADDRESS MAY BE DIFFERENT IN OTHER
;*       SYSTEMS.
;* ....................................................
;*
;*
;* (((  EQUATES  )))
;*
STATS   EQU  0FAH    ;@* KEYBOARD STATUS PORT
STROB   EQU  01      ;@* KEYBOARD STROBE BIT
DATA    EQU  0FCH    ; 'SOL' KEYBOARD DATA
VDM     EQU  0CC00H  ; 'SOL' BEGINNING OF SCREEN
VDMPG   EQU  0CCH    ; 'SOL' VDM RAM PAGE #
VDMPT   EQU  0FEH    ; 'SOL' VDM PORT ADDRESS
TARGT   EQU  05H     ; TARGET SYMBOL
RUNNR   EQU  07H     ; RUNNER SYMBOL
USLSH   EQU  2FH     ; UP SLASH CODE '/'
DSLSH   EQU  5CH     ; DOWN SLASH ' '
SOLOS   EQU  0C004H   ;__; SOLOS return address
CPM     EQU  00000H  ; CP/M RETURN ADDRESS
;
; - - - - - - - - - - - - - - - - - - - - - - - - - -
;
START:  LXI  SP,STACK+8
        CALL CLEAR         ; CLEAR SCREEN
REIDL:  CALL DRUNR         ; DISPLAY RUNNER
IDLE:   CALL ACTIN         ; MOVE RUNNER
        CALL DELAY         ; EX. TIME DELAY & CHECK INPUT
        MOV  A,C           ; KEYBD. INPUT TO ACCUMULATOR
        ORA  A             ; TEST FOR NON-ZERO
        JZ   IDLE
        MVI  M,20H         ; MOVE RUNNER
        RRC                ; SHIFT DEFLECTORS ?
        PUSH PSW           ; SAVE GAME OPTION IN A REG.
        JC   SHIFT         ; ...JUMP TO SHIFT IF YES
        CALL CLEAR         ; DON'T SHIFT, CLEAR SCREEN
        JMP  INIT
SHIFT:  LXI  B,VDM         ; SHIFT
        MOV  D,B           ;   SCREEN
        MOV  E,C           ;     ONE
        INX  D             ;       POSITION
SMOR:   LDAX D             ;         TO
        STAX B             ;           THE
        INX  B             ;             LEFT
        INX  D
        MOV  A,D
        CPI  VDMPG+4       ;@*
        JNZ  SMOR          ; GO SHIFT MORE IF NOT DONE
INIT:   POP  PSW           ; RESTORE GAME OPTION TO "A" REG
        MVI  C,1           ; SET FOR ONE TARGET
        RRC                ; GAME OPTION BIT 0 TO CARRY FLAG
        MVI  A,TARGT       ;@* TARGET SYMBOL TO "A" REG.
        JNC  GAM1          ; JUMP FOR 1 TARGET OPTION
        MVI  C,3           ; INITIALIZE FOR 3 TARGETS
        STA  VDM+034CH     ;@* PUT TARG LEFT OF CENTER
        STA  VDM+02F4H     ;@* TARGET TO THE RIGHT
GAM1:   STA  VDM+0320H     ;@* TARGET IN THE MIDDLE
        MOV  A,C
        STA  TARNM         ; STORE # OF TARGETS
        LXI  H,01C0H       ;@* INTIALIZE GAME TIMER
        SHLD TIMER         ; SAVE TIMER VALUE
        CALL DRUNR         ; START GAME-DISPLAY RUNNER
; * * * * * * * * * * * * * * * * * * * * * * * * * * *
;   THIS IS THE MAIN LOOP FOR THE GAME
;
MAINL:  CALL DELAY         ; WAIT AWHILE & CHK. KEYBD.
        CALL ACTIN
        PUSH H             ; SAVE "HL"
        LHLD TIMER         ; GET GAME TIMER
        DCX  H             ; DECREMENT TIMER
        MOV  A,H           ; CHECK IF GAME TIMER
        ORA  L             ; ...EXPIRED
        SHLD TIMER         ; STORE UPDATED TIMER
        POP  H             ; RESTORE "HL"
        JNZ  MAINL         ; JMP IF TIMER UNEXPIRED
        LXI  B,LOSE        ; ...DISPLAY NO
        CALL SHOW          ;   ...TIMER MESSAGE
        CALL WAER          ; WAIT; THEN ERASE MSG.
        JMP  START         ; TIMER EXPIRED, RESTART GAME
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE INITIALIZES AND DISPLAYS RUNNER
;          "B" REGISTER HOLDS MOTION DIRECTION - CODED AS:
;              REG. BIT  7 6 5 4 3 2 1 0
;                        | | | | . . . .
;                        | | | RIGHT
;                        | | UP
;                        | DOWN
;                        LEFT
;
DRUNR:  LXI  H,VDM+0040H   ;@* RUNNER STARTING POSITION
        LXI  D,0100H       ; X,Y RUNNER POSITION: D=VERT E=HORZ
        MVI  M,RUNNR       ;@*  DISPLAY RUNNER
        MVI  B,010H        ; INITIALIZE MOTION 2 RIGHT
        RET
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE DOES ALL THE RUNNER MOVEMENT
;
ACTIN:  MOV  A,M       ; CHECK IF
        CPI  RUNNR     ;@* .....RUNNER displayed
        JNZ  DF010
        MVI  M,20H      ; REMOVE RUNNER FROM FIELD
;
; -=<*>=- *   CALCULATE NEW RUNNER POSITION
;
DF010:  MOV  A,B
        CPI  010H       ; IS MOTION TO RIGHT ?
        JNZ  DF012      ; ...JUMP IF NO
        INR  E          ; MOVE ONE POSITION TO RIGHT
DF012:  CPI  020H       ; IS MOTION UP ?
        JNZ  DF014      ; ...JUMP IF NO
        DCR  D          ; MOVE 1 POSITION UP
DF014:  CPI  040H       ; IS MOTION DOWN ?
        JNZ  DF016      ; ...JUMP IF NO
        INR  D          ; MOVE 1 POSITION DOWN
DF016:  CPI  080H       ; IS MOTION TO LEFT ?
        JNZ  DF018      ; ...JUMP IF NO
        DCR  E          ; MOVE 1 POSITION TO LEFT
;
; -=<*>=- *   CALCULATES FIELD POSITION FROM HORIZONTAL
;             AND VERTICAL VALUES IN "D" AND "E" REG.;
;             RESULT IN "H" AND "L".
;
DF018:  MOV  A,D
        RRC
        RRC
        MOV  L,A
        ANI  03H
        ORI  VDMPG      ;@*
        MOV  H,A        ; SAVE HIGH PART OF ADDRESS
        MOV  A,L
        ANI  0C0H
        ADD  E
        MOV  L,A        ; SAVE LOW PART OF ADDRESS
;
        MOV  A,M        ; CHECK NEXT FIELD POSITION
        CPI  TARGT      ;@*  IS IT A TARGET ?
        JNZ  DF019      ; JUMP IF IT DIDN'T HIT A TARGET
;
; -=<*>=- *      HIT  TARGET
;
        MVI  M,20H      ; REMOVE TARGET
        LDA  TARNM      ; GET # OF TARGETS LEFT
        DCR  A          ; REDUCE # BY ONE
        STA  TARNM      ; STORE UPDATED TARGET COUNT
        RNZ             ; RETURN IF MORE TARGETS
        POP  D          ; ALL TARG GONE-FIX STACK P.
        LXI  B,WIN      ; DISPLAY WIN MESSAGE
        CALL SHOW
        CALL WAER       ; WAIT AWHILE; THEN ERASE MSG.
        JMP  REIDL      ; GOTO IDLE LOOP
;
DF019:  MOV  A,C        ; PUT KEYBD. DATA INTO A REG.
        CPI  USLSH      ; IS IT AN UP SLASH ? '/'
        JZ   DF020      ; ...JUMP IF YES
        CPI  DSLSH      ; IS IT A DOWN SLASH ? ' '
        JZ   DF020      ; ...JMP 2 ANALYZE DFLEK IF GOOD IN
;
; -=<*>=- *    CHECK FOR RUNNER HITTING A DEFLECTOR
;
DFLCK:  MOV  A,M
        CPI  20H      ; IS NEXT POSITION BLANK ?
        JNZ  CHGMT    ; DEFLECTOR IF NO-CHANGE DIRECTION
        CALL EDGCK    ; REVERSE MOTION IF AT END OF FIELD
        MVI  M,RUNNR  ;@*  DISPLAY RUNNER AT NEW LOCATION
        RET
DF020:  MOV  M,A      ; INPUT WAS VALID: DISPLAY IT
;
; -=<*>=- *   CHANGE DIRECTION OF RUNNER MOTION
;
CHGMT:  CPI  DSLSH    ; IS IT A DOWN SLASH ?
        MOV  A,B      ; DIRECTION FLAG TO REG.
        JNZ  UPSLH    ; JUMP IF NOT DOWN SLASH
        ANI  0C0H     ; IS MOTION LEFT OR DOWN ?
        MOV  A,B      ; RESTORE DIRECTION FLAG TO "A" REG.
        JZ   DF030    ; JUMP IF NOT LEFT OR DOWN
        RRC           ; CHANGE DIRETION TO RIGHT OR UP
        RRC
        JMP  DF050
DF030:  RLC           ; CHANGE DIRECTION TO LEFT OR DOWN
        RLC
        JMP  DF050
UPSLH:  ANI  0A0H     ; IS MOTION TO LEFT OR UP ?
        MOV  A,B      ; DIRECTION FLAG TO "A" REG.
        JZ   DF040    ; JUMP IF NOT LEFT OR UP
        RRC           ; CHANGE DIRECTION TO RIGHT OR DOWN
        JMP  DF050
DF040:  RLC           ; CHANGE MOTION TO RIGHT OR DOWN
DF050:  MOV  B,A      ; SAVE NEW DIRECTION
        CALL EDGCK    ; IS RUNNER AT EDGE OF FIELD ?
        MOV  A,M      ; GET DEFLECTOR IN CASE AT FIELD EDGE
        JZ   CHGMT    ; IF AT FIELD EDGE-CHG DIRECTION AGAIN
        RET
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE CHECKS FOR EDGE OF FIELD, REVERSES MOTION
;      AND SETS ZERO FLAG IF AT EDGE.
;
EDGCK:  MOV   A,B     ; LOAD MOTION DIRECTION FOR TESTING
        CPI   010H    ; IS MOTION TO RIGHT ?
        JNZ   DF060   ; ...JUMP IF NO
        MVI   A,3FH   ; FOR RIGHT EDGE HORZ='3F'
        CMP   E
        RNZ           ; RETURN IF NOT AT RIGHT EDGE
        MVI   B,080H  ; REVERSE DIRECTION TO LEFT
        RET
DF060:  CPI   020H    ; CHECK FOR MOTION UP
        JNZ   DF070   ; JUMP IF MOTION NOT UP
        XRA   A
        ORA   D       ; CHECK FOR TOP - VERT=0
        RNZ           ; RETURN IF NOT AT TOP
        MVI   B,040H  ; REVERSE DIRECTION TO DOWN
        RET
DF070:  CPI   040H    ; CHECK FOR MOTION DOWN
        JNZ   DF080   ; ...JUMP IF MOTION NOT DOWN
        MVI   A,0FH   ; CHECK FOR BOTTOM - VERT='0F'
        CMP   D
        RNZ           ; RETURN IF NOT AT BOTTOM
        MVI   B,020H  ; REVERSE DIRECTION TO UP
        RET
DF080:  XRA   A       ; MUST BE MOTION TO LEFT
        ORA   E       ; CK FOR LEFT EDGE - HORZ=0
        RNZ           ; RETURN IF NOT LEFT EDGE
        MVI   B,010H    ; REVERSE DIRECTION TO RIGHT
        RET
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE PERFORMS A TIME DELAY WHILE CONTINUOUSLY
;   CHECKING FOR KEYBOARD INPUT - INPUT RETURN IN "C" REG.
;
DELAY:  MVI  C,0      ; INITIALIZE INPUT SAVE REG.
        PUSH D        ; FREE UP "D" & "E" REGS.
TDL:    LXI  D,0900H  ; LOAD DELAY VALUE
DL01:   DCX  D        ; DECREMENT DELAY COUNT
        IN   STATS    ;@* GET KEYBOARD STATUS
        ANI  STROB    ;@* CHECK STROBE
        JNZ  DL02     ;@* ...JUMP IF NO INPUT
        IN   DATA     ;@* GET KEYBOARD DATA
        CPI  01BH      ;__*   "ESC" key ?
       IF   USECPM
        JZ   CPM       ;__* ...yes, return to CP/M
       ENDIF
       IF   NOT USECPM
        JZ   SOLOS     ;__* ...yes, return to SOLOS
       ENDIF
        ANI  7FH      ;   REMOVE BIT 7
        MOV  C,A      ;   SAVE INPUT IN "C" REG.
DL02:   MOV  A,D      ;   CHECK FOR TIMER = 0
        ORA  E
        JNZ  DL01     
        POP  D        ; DELAY OVER, RESTORE "D" & "E" REG.
        RET
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE INITIALIZES THE VDM PORT & CLEARS THE SCREEN
;
CLEAR:  XRA  A
        OUT  VDMPT      ;@* CLEAR VDM PORT
        LXI  H,VDM      ;@* POINT TO START OF VDM RAM
        MVI  A,VDMPG+4  ; "A" REG.=END OF VDM RAM
CL01:   MVI  M,20H      ; BLANK LOCATION GIVEN BY "HL"
        INX  H
        CMP  H          ; DONE ?
        JNZ  CL01       ; ...JUMP IF NOT
        RET
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE DISPLAYS MESSAGE POINTED TO BY "BC" REG.
;
SHOW:   LXI  D,VDM+031DH  ;@* POINT TO DISPLAY LOCATION
        MVI  H,7          ; MESSAGE LENGTH FIXED AT 7
SH01:   LDAX B            ; GET BYTE OF MESSAGE
        STAX D            ; DISPLAY IT
        INX  D            ; BUMP TO NEXT BYTE
        INX  B
        DCR  H
        JNZ  SH01         ; DONE ?
        RET
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; THIS ROUTINE WAITS AWHILE & THEN REMOVES MESSAGE
;
WAER:    MVI  D,32       ; LOAD UNITS OF DELAY
WDLY:    CALL DELAY      ; WAIT ONE DELAY UNIT
         DCR  D          ; CHECK IF DONE WAITING
         JNZ  WDLY
         LXI  B,BLNK     ; POINT TO ALL BLANKS
         JMP  SHOW
;
; MESSAGES
;
WIN:    DB  'WINNER '
LOSE:   DB  'NO TIME'
BLNK:   DB  '       '
;
;  SAVE AREAS AND STACK
;
TIMER:  DW 0       ;TIMER SAVE AREA
TARNM:  DB 0       ; # OF TARGETS SAVE AREA
STACK:  DS 8       ; STACK AREA ( ONLY 4 DEEP )
TDLAY   EQU  TDL+1 ; game SPEED / TIME DELAY val.
ENDIT:  END

